/*
 * shared_memory.c
 *
 *  Created on: Aug 2, 2014
 *      Author: andreas
 */

#include "shared_memory.h"

int32_t shm_id;

void setup_shared_memory()
{
	//calculate size of shared memory

	size_t shm_size = SIM_STATION_COUNT * (11 * sizeof(uint32_t) + SIM_PHY_BUFFER_COUNT * SIM_PHY_BUFFER_SIZE) + sizeof(int) + HISTORY_SIZE * sizeof(historyItem) + SIM_STATION_COUNT  * sizeof(historyItem) + sizeof(uint32_t) + SIM_STATION_COUNT * (sizeof(int) + MARKERS_SIZE * sizeof(Marker)) + (sizeof(int) + MARKERS_SIZE * sizeof(Marker)) + SIM_STATION_COUNT * sizeof(Stats);

	//try 3 times to allocate shared memory with random key

	srand(time(NULL));
	key_t key = rand();

	int try = 0;
	while (try < 3 && (shm_id = shmget( key, shm_size, IPC_CREAT | IPC_EXCL | 0666 )) < 0)
	{
		key = rand();
		try++;
	}

	if(shm_id < 0)
	{
		printf("SIM_ERROR: Couldn't allocate shared memory.\n");
		main_exit(1);
	}

	//attach shared memory

	void *shm = 0;
	if ((shm = shmat( shm_id, NULL, 0 )) == 0)
	{
		printf("SIM_ERROR: Couldn't attach shared memory.\n");
		release_shared_memory();
		main_exit(1);
	}

	memset(shm, 0, shm_size);

	//initialize pointers

	sim_left_buttons = (uint32_t*)((void*)shm);
	sim_right_buttons = (uint32_t*)((void*)sim_left_buttons + SIM_STATION_COUNT * sizeof(uint32_t));
	sim_up_buttons = (uint32_t*)((void*)sim_right_buttons + SIM_STATION_COUNT * sizeof(uint32_t));
	sim_middle_buttons = (uint32_t*)((void*)sim_up_buttons + SIM_STATION_COUNT * sizeof(uint32_t));

	sim_rx_states = (uint32_t*)((void*)sim_middle_buttons + SIM_STATION_COUNT * sizeof(uint32_t));
	sim_tx_states = (uint32_t*)((void*)sim_rx_states + SIM_STATION_COUNT * sizeof(uint32_t));

	sim_rx_buffer_ids = (uint32_t*)((void*)sim_tx_states + SIM_STATION_COUNT * sizeof(uint32_t));
	sim_tx_buffer_ids = (uint32_t*)((void*)sim_rx_buffer_ids + SIM_STATION_COUNT * sizeof(uint32_t));

	sim_rx_header_indicators = (uint32_t*)((void*)sim_tx_buffer_ids + SIM_STATION_COUNT * sizeof(uint32_t));
	sim_rx_payload_indicators = (uint32_t*)((void*)sim_rx_header_indicators + SIM_STATION_COUNT * sizeof(uint32_t));
	sim_tx_active_indicators = (uint32_t*)((void*)sim_rx_payload_indicators + SIM_STATION_COUNT * sizeof(uint32_t));

	sim_packet_buffers = (void**)((void*)sim_tx_active_indicators + SIM_STATION_COUNT * sizeof(uint32_t));

	carrier_sense = (void*)((void*)sim_packet_buffers + SIM_STATION_COUNT * SIM_PHY_BUFFER_COUNT * SIM_PHY_BUFFER_SIZE);

	history_last_item = (void*)((void*)carrier_sense + sizeof(uint32_t));
	history = (void*)((void*)history_last_item + sizeof(int));
	station_current_history_items  = (void*)((void*)history + HISTORY_SIZE * sizeof(historyItem));

	marker_last_items = (int*)((void*)station_current_history_items + SIM_STATION_COUNT * sizeof(historyItem));
	markers = (Marker**)((void*)marker_last_items + SIM_STATION_COUNT * sizeof(int));

	csma_marker_last_item = (int*)((void*)markers + SIM_STATION_COUNT * MARKERS_SIZE * sizeof(Marker));
	csma_marker = (Marker*)((void*)csma_marker_last_item + sizeof(int));

	l_stats = (void*)((void*)csma_marker + MARKERS_SIZE * sizeof(Marker));

	//check for correct pointer initialization
	void *shm_end = (void*)((void*)l_stats + SIM_STATION_COUNT * sizeof(Stats));

	if(shm_end - shm != shm_size)
	{
		printf("SIM_ERROR: Shared memory pointer mapping faulty.\n");
		release_shared_memory();
		main_exit(1);
	}

	printf("SIM_INFO: Shared memory initialized. Key: 0x%x. Size: %lu.\n", key, shm_size);

}

void release_shared_memory()
{
	if(shmctl(shm_id, IPC_RMID, NULL) < 0)
	{
		printf("SIM_ERROR: Couldn't release shared memory.\n");
	}
	else
	{
		printf("SIM_INFO: Shared memory released.\n");
	}
}
