/*
 * station.c
 *
 *  Created on: Aug 2, 2014
 *      Author: andreas
 */

#include "station.h"
#include "shared_memory.h"
#include "warpphy.h"

void station_signal_handler(int signal);

void connect_pty()
{
	int slave;
	if(openpty(&station_pty_fd, &slave, NULL, NULL, NULL) != 0)
	{
		printf("STATION_ERROR(%d): Couldn't open PTY.\n", station_id);
	}
	else
	{
		//turn off echoing

		struct termios	stermios;

		if (tcgetattr(station_pty_fd, &stermios) < 0)
		{
			printf("STATION_ERROR(%d): tcgetattr failed.\n", station_id);
		}

		stermios.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL);
		//stermios.c_oflag &= ~(ONLCR);

		if (tcsetattr(station_pty_fd, TCSANOW, &stermios) < 0)
		{
			printf("STATION_ERROR(%d): tcsetattr failed.\n", station_id);
		}

		//open as file for fprintf

		station_pty_file = fdopen(station_pty_fd, "w");

		long flag = fcntl(station_pty_fd, F_GETFL, 0 );
		fcntl(station_pty_fd,F_SETFL,flag | O_NONBLOCK);

		printf("STATION_INFO(%d): Connected PTY: %s\n", station_id, ptsname(station_pty_fd));
	}
}

void connect_eth()
{
	sockfd = socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);

	long flag = fcntl(sockfd, F_GETFL, 0 );
	fcntl(sockfd,F_SETFL,flag | O_NONBLOCK);

	/*int on = 1;
	setsockopt(sockfd, IPPROTO_IP, IP_HDRINCL, &on, sizeof(on));*/

	station_udp_port = 7500;

	bzero(&station_ip_addr,sizeof(station_ip_addr));
	station_ip_addr.sin_family = AF_INET;
	station_ip_addr.sin_addr.s_addr=inet_addr("127.0.0.0");
	station_ip_addr.sin_addr.s_addr += 100 + (station_id << 24);
	station_ip_addr.sin_port= htons(station_udp_port);

	int bind_return = bind(sockfd,(struct sockaddr *)&station_ip_addr,sizeof(station_ip_addr));

	if(bind_return == -1)
	{
		printf("socket error: %d, %s\n", errno, strerror(errno));
		return;
	}

	printf("opened socket: %s:%d\n", inet_ntoa(station_ip_addr.sin_addr), station_udp_port);
}

int station_main(int id)
{
	station_id = id;

	struct sigaction action;
	memset(&action, 0, sizeof(struct sigaction));
	action.sa_handler = station_signal_handler;
	sigaction(SIGTERM, &action, NULL);
	sigaction(SIGINT, &action, NULL);
	sigaction(SIGQUIT, &action, NULL);
	sigaction(SIGHUP, &action, NULL);

	//connect_pty();
	connect_eth();

	station_led_low = 0;
	station_led_high = 0;

	station_left_hex = 0;
	station_middle_hex = 0;
	station_right_hex = 0;

	station_rx_header_indicator = ((void*)sim_rx_header_indicators) + station_id * sizeof(uint32_t);
	station_rx_payload_indicator = ((void*)sim_rx_payload_indicators) + station_id * sizeof(uint32_t);

	station_tx_active_indicator = ((void*)sim_tx_active_indicators) + station_id * sizeof(uint32_t);

	station_rx_buffer_id = ((void*)sim_rx_buffer_ids) + station_id * sizeof(uint32_t);
	station_tx_buffer_id = ((void*)sim_tx_buffer_ids) + station_id * sizeof(uint32_t);

	station_packet_buffers = ((void*)sim_packet_buffers) + station_id * SIM_PHY_BUFFER_COUNT * SIM_PHY_BUFFER_SIZE;

	station_left_button = ((void*)sim_left_buttons) + station_id * sizeof(uint32_t);
	station_right_button = &(sim_right_buttons[station_id]);// + station_id * sizeof(uint32_t);
	station_up_button = ((void*)sim_up_buttons) + station_id * sizeof(uint32_t);
	station_middle_button = ((void*)sim_middle_buttons) + station_id * sizeof(uint32_t);

	lock_semaphore(SEM_SIM_START);
	unlock_semaphore(SEM_SIM_START);

	usleep(rand() % 1000 * SIM_SLOWDOWN);

	mac_main();

	return 0;
}

void station_signal_handler(int signal)
{
	const char *signal_name;

	switch (signal) {
		case SIGTERM:
			signal_name = "SIGTERM";
			break;
		case SIGQUIT:
			signal_name = "SIGQUIT";
			break;
		case SIGINT:
			signal_name = "SIGINT";
			break;
		case SIGHUP:
			signal_name = "SIGHUP";
			break;
		default:
			signal_name = "UNKNOWN_SIGNAL";
			break;
	}

	printf("STATION_INFO(%d): Received %s, exiting...\n", station_id, signal_name);

	exit(0);
}
