/**
 * @file	mnp.h
 * @author	cd185990
 * @date	2010/06/26
 * @brief	MNP Group Declaration / Interface File
 *
 * Last Change Base: 		$LastChangedRevision: 444 $
 * Last Changed By: 		$LastChangedBy: cd185990 $
 * Last Changed On: 		$LastChangedDate: 2014-02-14 12:23:46 +0100 (Fri, 14 Feb 2014) $
 *
 * Copyright: 2010 - 2014, Mobile Network Performance Group, RWTH Aachen
 */

#include "xparameters.h"
#include "limits.h"
#include "stdio.h"

#include "warpmac.h"
#include "warpphy.h"

#ifndef MNP_H_
#define MNP_H_


/* ********************************************
 * General defines
 *********************************************/
#define MNP_VERSION					"0.3.13"
#define CALLBACK_OK					0
#define CALLBACK_ERROR				-1

#define W1_MAC_ADDR					{0x00,0x1D,0x72,0x86,0xF2,0xE3}
#define W2_MAC_ADDR					{0x00,0x22,0x68,0x14,0xF1,0x66}
#define W3_MAC_ADDR					{0x00,0x24,0x7E,0xDB,0x09,0xBE}
#define W4_MAC_ADDR					{0x00,0x21,0x86,0x9F,0x39,0xF7}
//arp -s 192.168.1.3 00-24-7e-db-09-be
//arp -s 192.168.1.4 00-1d-72-86-f2-e3

#ifndef max
	#define max( a, b ) ( ((a) > (b)) ? (a) : (b) )
#endif

#ifndef min
	#define min( a, b ) ( ((a) < (b)) ? (a) : (b) )
#endif

#ifndef even
	#define even( a ) ( ((a)%2 == 0) ? 1 : 0)
#endif

#ifndef ceil
	#define ceil( a ) ( (a)<0 ? (int)(a) : (((a)-(int)(a)) == 0 ? (int)(a) : (int)(a)+1 ) )
#endif

#ifndef floor
	#define floor( a ) ( (a)<0 ? (int)(a)-1 : (int)(a) )
#endif

#ifndef notneg
	#define notneg( a ) ( (a)<0 ? 0 : (a) )
#endif


/* ********************************************
 * Timer
 *********************************************/

/**
 * @brief Bypass WARPMAC timer functions due to their built-in stuff
 *
 * @param timer
 * @param length in us
 * @return CALLBACK_OK || CALLBACK_ERROR
 */
int	mnp_timer_set(unsigned char timer, unsigned int length);

/**
 * @brief Bypass WARPMAC timer functions due to their built-in stuff
 *
 * @param timer
 * @return CALLBACK_OK || CALLBACK_ERROR
 */
int	mnp_timer_remove(unsigned char timer);

/**
 * @brief Bypass WARPMAC timer functions due to their built-in stuff
 *
 * @param timer
 * @return CALLBACK_OK || CALLBACK_ERROR
 */
int	mnp_timer_pause(unsigned char timer);

/**
 * @brief Bypass WARPMAC timer functions due to their built-in stuff
 *
 * @param timer
 * @return CALLBACK_OK || CALLBACK_ERROR
 */
int	mnp_timer_resume(unsigned char timer);

/**
 * @brief Bypass WARPMAC timer functions due to their built-in stuff
 *
 * @param timer
 * @return if timer is active or not
 */
unsigned char	mnp_timer_isActive(unsigned char timer);

/**
 * @brief Bypass WARPMAC timer functions due to their built-in stuff
 *
 * @param timer
 * @return if timer is finished or not
 */
unsigned char	mnp_timer_isDone(unsigned char timer);

/**
 * @brief Bypass WARPMAC timer functions due to their built-in stuff
 *
 * @param timer
 * @return if timer is finished or not
 */
unsigned char	mnp_timer_isPaused(unsigned char timer);

/* ********************************************
 * DebugGPIO
 *********************************************/

/**
 * @brief Set DebugGPIO pins active.
 *
 * @param bitmask
 */
void	mnp_debuggpio_set(unsigned char bitmask);

/**
 * @brief Set DebugGPIO pins inactive.
 *
 * @param bitmask
 */
void	mnp_debuggpio_unset(unsigned char bitmask);

#ifdef XPAR_MNP_GPIOCONNECT_PLBW_NUM_INSTANCES

/**
 * @brief Set DebugGPIO pins inactive.
 *
 * @return status of connected DebugGPIO pin (1/0)
 */
unsigned char	mnp_debuggpio_get();

#else

#define mnp_debuggpio_get() 0

#endif

/* ********************************************
 * Counter
 *********************************************/
#define MNP_COUNTER_MAX			ULONG_LONG_MAX
#define MNP_COUNTER_SCALE		80				//(80000000/1000000)
#define MNP_COUNTER_RESET_BIT	0x01
#define MNP_COUNTER_ENABLE_BIT	0x02

#ifdef XPAR_MNP_COUNTER_PLBW_NUM_INSTANCES

void	mnp_counter_start();
unsigned long long mnp_counter_stop();
unsigned long long mnp_counter_getValue();

#else /* XPAR_MNP_COUNTER_PLBW_NUM_INSTANCES */

#define mnp_counter_start() {}
#define mnp_counter_stop() MNP_COUNTER_MAX
#define mnp_counter_getValue() MNP_COUNTER_MAX

#endif /* XPAR_MNP_COUNTER_PLBW_NUM_INSTANCES */


#endif /* MNP_H_ */
