/**
 * @file	sync.c
 * @author	cd185990
 * @date	2011/12/08
 * @brief	Synchronization Interface File
 *
 * Last Change Base: 		$LastChangedRevision: 364 $
 * Last Changed By: 		$LastChangedBy: cd185990 $
 * Last Changed On: 		$LastChangedDate: 2013-09-02 18:03:17 +0200 (Mon, 02 Sep 2013) $
 *
 * Copyright: 2010 - 2013, Mobile Network Performance Group, RWTH Aachen
 */

#include "sync.h"
#include "debug.h"
#ifndef DEBUG_MSGLEVEL
	#define DEBUG_MSGLEVEL DBG_CRIT
#endif
#include "network.h"
#include "xparameters.h"

#if defined XPAR_MNP_SYNC_PLBW_NUM_INSTANCES

#include "limits.h"
#include "xbasic_types.h"
#include "xstatus.h"
#include "xio.h"
#include "xgpio.h"
#include "xllfifo.h"
#include "xllfifo_hw.h"
#include "xlltemac.h"

bool mnp_sync_waitForSync() {
	bool wasSync = mnp_sync_isSynchronized();
	if (wasSync == true)
		mnp_sync_reset();

	debug(DBG_INFO,"%s\r\n","Disabled packet reception and dummyPacketMode until synchronization packet is received.");
//	warpmac_setDummyPacketMode(0);
	unsigned char enabledEthernet = warpmac_isDataFromNetworkEnabled();
	warpmac_disableDataFromNetwork();

	while (true) {
		// is packet in buf?
		if (XIo_In32(warpmac_getEMACbaseAddress()+XLLF_RDFO_OFFSET) > 0) {
			//Clear the LLFIFO status bits
			XIo_Out32(warpmac_getEMACbaseAddress()+XLLF_ISR_OFFSET, 0x0FFFFFFF);
			//
			emacRx_handler(1); // special handling of packets while in inSyncMode
			void *payload = warpmac_getEMACPayload(warpmac_getEMACRxBuffer());

			// reuse packet
			ethhdr* eth = (ethhdr *)payload;
			if (eth->eth_proto == ETH_TYPE_IP4) {
				iphdr* ip = (iphdr *)(payload + ETH_HDR_LENGTH);
				if (ip->ip_dst.s_b4 == 255 && ip->ip_p == IP_P_UDP) {
					// UDP Packet distinction
					udphdr* udp = (udphdr *)(payload + ETH_HDR_LENGTH + IP_HDR_LENGTH);
					if (udp->uh_sport == mnp_sync_getServerPort() && (udp->uh_ulen >= (UDP_HDR_LENGTH + MNP_SYNC_SUBHEADER_LENGTH))) { // sanity check
						syncPacketHeader* packetHeader = (syncPacketHeader *)(payload + ETH_HDR_LENGTH + IP_HDR_LENGTH + UDP_HDR_LENGTH);
						// check if correct header and correct packet type
						if ((packetHeader->identifier != MNP_SYNC_IDENTIFIER) || packetHeader->type != WarpDiscovery || udp->uh_dport != mnp_sync_getPort(WarpDiscovery))
							continue;
#ifdef DEBUG
						if (XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_CONFIG_N_BITS != 2) {
							debug(DBG_FATAL,"%s\r\n","Mismatching hardware and software.");
							warpmac_leftHex(0xE);
							warpmac_rightHex(0xE);
							return false;
						}
#endif
						// start pcore
						mnp_sync_start();

						/// answer packet generation
						// SYNC
						syncPacketWarpDiscoveryAndSynchronization packet;
						packet.subheader.identifier = (0xDEAD^0xBEEF);
						packet.subheader.padding = 0xAB;
						packet.subheader.type = WarpDiscoveryAndSynchronization;
						memmove(packetHeader,&packet,MNP_SYNC_SUBHEADER_LENGTH);

						// UDP
						udp->uh_ulen = UDP_HDR_LENGTH + MNP_SYNC_SUBHEADER_LENGTH;
						udp->uh_dport = udp->uh_sport;
						udp->uh_sport = mnp_sync_getPort(WarpDiscoveryAndSynchronization);
						//udp->uh_sport = wasSync == true ? mnp_sync_getPort(WarpDiscovery) : mnp_sync_getPort(WarpDiscoveryAndSynchronization);
						udp->uh_sum = 0x0000;

						// IP
						ip->ip_len = IP_HDR_LENGTH + udp->uh_ulen;
						memmove(&(ip->ip_dst),&(ip->ip_src),4);
						ip->ip_src.s_b4 = mnp_sync_getNodeExternalID(warpmac_getMyId()); // spoof same IP range, but change last part
						ip->ip_sum = 0; // checksum calc requires ip_sum == 0
						int summ=0; // IP checksum calc
						int m=0;
						unsigned short vari = 0;
						//unsigned char* temp = payload + ETH_HDR_LENGTH;
						for (int j=0;j<10;j++) {
							memmove(&vari,payload+14+m,2);
							summ=summ+vari;
							m=m+2;
						}
						unsigned short one = 0;
						unsigned int two = 0;
						memmove(&one,&summ,2);
						two = summ-(one*65536);
						unsigned short one_two = ~(one+two);
						memmove(&(ip->ip_sum),&one_two,2);

						// ETH
						memmove(eth->eth_dest,eth->eth_src,6);
						unsigned char temp[] = MNP_SYNC_MAC_BASE;
						memmove(eth->eth_src,temp,6);
						eth->eth_src[5] = mnp_sync_getNodeExternalID(warpmac_getMyId());

//						usleep((int)(rand()%10 + 1)*1000*warpmac_getMyId()); // avoid collisions
						warpmac_prepPktToNetwork(warpmac_getEMACPayload(warpmac_getEMACRxBuffer()), (ETH_HDR_LENGTH + ip->ip_len));
						warpmac_startPktToNetwork((ETH_HDR_LENGTH + ip->ip_len));

						break;
					}
				}
			}
		}
	}

	if (enabledEthernet) warpmac_enableDataFromNetwork();

	return !wasSync;
}

bool mnp_sync_isSynchronized() {
	if (XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_RUNNING_N_BITS != 1) {
		debug(DBG_FATAL,"%s\r\n","Mismatching hardware and software.");
		return false;
	}
	return XIo_In32(XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_RUNNING) & 0x1 ? true : false;
}

unsigned long long mnp_sync_getValue() {
	unsigned long long value = XIo_In32(XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_VALUE_H);
	value = (value << 32) | (unsigned long long)XIo_In32(XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_VALUE_L); // 2-cycle register read
	if ((value >> 32) == 0xFFFFFFFF && XIo_In32(XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_VALUE_H) == 0) { // check for high-byte wrap-around and correct values
		value &= 0x0000FFFF;
	}
	return value;
}

inline void mnp_sync_start() {
	XIo_Out32(XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_CONFIG,0x00);
	XIo_Out32(XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_CONFIG,MNP_SYNC_MASK_START);
}

inline void mnp_sync_reset() {
	XIo_Out32(XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_CONFIG,0x00);
	XIo_Out32(XPAR_MNP_SYNC_PLBW_0_MEMMAP_SYNC_CONFIG,MNP_SYNC_MASK_RESET);
}

#endif
