/**
 * @file	sync.h
 * @author	cd185990
 * @date	2011/12/08
 * @brief	Synchronization Interface File
 *
 * Last Change Base: 		$LastChangedRevision: 411 $
 * Last Changed By: 		$LastChangedBy: cd185990 $
 * Last Changed On: 		$LastChangedDate: 2013-10-31 16:50:09 +0100 (Thu, 31 Oct 2013) $
 *
 * Copyright: 2010 - 2013, Mobile Network Performance Group, RWTH Aachen
 */

#ifndef SYNC_H_
#define SYNC_H_

#include "xparameters.h"
#include "stdbool.h"

#include "warpmac.h"
#include "warpphy.h"

#define MNP_SYNC_VERSION			"0.3.5" // requires sync pcore 1.01c

#define MNP_SYNC_PORT_SERVER		7500
#define MNP_SYNC_PORT_BASE			15000
#define MNP_SYNC_IP_OFFSET			100
#define MNP_SYNC_MAC_BASE			{0x00,0x50,0xc2,0x63,0x31,0x00}
#define MNP_SYNC_COUNTER_MAX		ULONG_LONG_MAX
#define MNP_SYNC_COUNTER_SCALE		80//(80000000/1000000) // result in us
#define MNP_SYNC_MASK_RESET			0x01
#define MNP_SYNC_MASK_START			0x02
#define MNP_SYNC_SUBHEADER_LENGTH	4
#define MNP_SYNC_IDENTIFIER			(0xDEAD^0xBEEF)
#define MNP_SYNC_PACKETTYPES		6

typedef enum {
	Invalid,
	WarpDiscovery,
	WarpDiscoveryAndSynchronization,
	WarpRequestStatus,
	WarpGetStatus,
	WarpStatus
} syncPacketType;

typedef struct {
	unsigned short int identifier;
	unsigned char padding; // WARP struct padding
	unsigned char type; // 4 Byte full
} syncPacketHeader;

typedef struct {
	syncPacketHeader subheader;
} syncPacketWarpDiscovery, syncPacketWarpDiscoveryAndSynchronization;

typedef struct {
	syncPacketHeader subheader;
	unsigned char single;
} syncPacketNodeGetStatus, syncPacketNodeRequestStatus;

typedef struct {
	syncPacketHeader subheader;
	unsigned char timersActive;
	unsigned char ringId;
	unsigned char selfRing;
	unsigned char successorId; // 4 Byte full
	unsigned char predecessorId;
	unsigned char state;
	unsigned char oldState;
	unsigned char lastTokenSent; // 4 Byte full
	unsigned char lastTokenReceived;
	unsigned char seq;
	unsigned char genSeq;
	unsigned char nodes; // 4 Byte full
	unsigned char oldSeq;
	unsigned char oldGenSeq;
	unsigned char oldNodes;
	signed char solicit; // 4 Byte full
	unsigned int errorCode; // 4 Byte full
	unsigned char tempVal[4]; // 4 Byte full
	unsigned char entries[45*32];
} syncPacketNodeStatus;

#if defined XPAR_MNP_SYNC_PLBW_NUM_INSTANCES

inline void mnp_sync_start();
inline void mnp_sync_reset();
bool mnp_sync_waitForSync();
bool mnp_sync_isSynchronized();
unsigned long long mnp_sync_getValue();
#define mnp_sync_getNodeExternalID(x) ((x) + MNP_SYNC_IP_OFFSET)
#define mnp_sync_getNodeInternalID(x) ((x) - MNP_SYNC_IP_OFFSET)

#else

#define mnp_sync_start() (0)
#define mnp_sync_reset() (0)
#define mnp_sync_waitForSync() (0)
#define mnp_sync_isSynchronized() (0)
#define mnp_sync_getValue() (0)
#define mnp_sync_getNodeExternalID(x) (x)
#define mnp_sync_getNodeInternalID(x) (x)

#endif

#define mnp_sync_getPort(x)		(MNP_SYNC_PORT_BASE + (x))
#define mnp_sync_getServerPort()	(MNP_SYNC_PORT_SERVER)

#endif /* SYNC_H_ */
