/*
 * main.c
 *
 *  Created on: Aug 2, 2014
 *      Author: andreas
 */

#include "main.h"

int viewer_pid;
int console_pid;
//int statistics_pid;
int station_pids[SIM_STATION_COUNT];

void main_signal_handler(int signal);

int main()
{
	//setup SIGTERM/SIGINT handler
	struct sigaction action;
	memset(&action, 0, sizeof(struct sigaction));
	action.sa_handler = main_signal_handler;
	sigaction(SIGTERM, &action, NULL);
	sigaction(SIGINT, &action, NULL);
	sigaction(SIGQUIT, &action, NULL);
	sigaction(SIGHUP, &action, NULL);

	time_program_launch = 0;
	time_program_launch = get_time_nsec();

	//setup shared memory
	setup_shared_memory();
	setup_semaphores();
	init_ipc();



	lock_semaphore(SEM_SIM_START);

	//fork viewer to child process
	if((viewer_pid = fork()) == 0)
	{
		exit(viewer_main());
	}

	printf("pipes: ");

	//create pipes
	int i;
	for(i=0; i<SIM_STATION_COUNT; ++i)
	{
		int fd[2];
		if (pipe (fd) < 0)
		{
			printf("SIM_ERROR: Couldn't create pipes.\n");
			main_exit(1);
		}

		uart_write_fds[i] = fd[1];
		uart_read_fds[i] = fd[0];

		printf("(%d,%d)", fd[0], fd[1]);
	}

	printf("\n");

	//fork console
	if((console_pid = fork()) == 0)
	{
		for(i=0; i<SIM_STATION_COUNT; ++i)
		{
			long flag = fcntl(uart_read_fds[i], F_GETFL, 0 );
			fcntl(uart_read_fds[i],F_SETFL,flag | O_NONBLOCK | O_NDELAY);

			close(uart_write_fds[i]);
		}

		exit(console_main());
	}

	/* disable fork statistics
	if((statistics_pid = fork()) == 0)
	{
		exit(statistics_main());
	}*/

	for(i=0; i<SIM_STATION_COUNT; ++i)
	{
		close(uart_read_fds[i]);
	}

	//fork stations


	for(i=0; i<SIM_STATION_COUNT; ++i)
	{
		//usleep(rand()%1000); //ms
		if((station_pids[i] = fork()) == 0)
		{
			// establish pipe-based communication between boards
			uart_write = fdopen(uart_write_fds[i], "w");

			int j;
			for(j=0; j<SIM_STATION_COUNT; ++j)
			{
				if(j != i)
				{
					close(uart_write_fds[j]);
				}
			}

			exit(station_main(i));
		}
	}

	int rv = simulator_main();

	main_signal_handler(SIGTERM);
	return rv;
}

void main_exit(int errval)
{
	main_signal_handler(SIGTERM);
	exit(errval);
}

void kill_stations()
{
	int i, pid ,status;
	for(i = 0; i < SIM_STATION_COUNT; ++i)
	{
		kill(station_pids[i], SIGKILL);

		pid = wait(&status);
		printf("SIM_INFO: Station with PID %d exited with status 0x%x.\n", pid, status);
	}
}

void main_signal_handler(int signal)
{
	const char *signal_name;
	
	switch (signal) {
		case SIGTERM:
			signal_name = "SIGTERM";
			break;
		case SIGQUIT:
			signal_name = "SIGQUIT";
			break;
		case SIGINT:
			signal_name = "SIGINT";
			break;
		case SIGHUP:
			signal_name = "SIGHUP";
			break;
		default:
			signal_name = "UNKNOWN_SIGNAL";
			break;
	}
	
	printf("SIM_INFO: Received %s, exiting...\n", signal_name);

	release_shared_memory();
	release_semaphores();
	release_ipc();

	kill(viewer_pid, SIGKILL);
	kill(console_pid, SIGKILL);
	// kill(statistics_pid, SIGKILL);

	int status;
	int pid = wait(&status);
	printf("SIM_INFO: Viewer with PID %d exited with status 0x%x.\n", pid, status);

	kill_stations();

	exit(0);
}
