/*
 * shared_memory.h
 *
 *  Created on: Aug 2, 2014
 *      Author: andreas
 */

#ifndef SHARED_MEMORY_H_
#define SHARED_MEMORY_H_

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <time.h>

#include "main.h"
#include "settings.h"
#include "warpmac.h"
#include "warpphy.h"

typedef struct history_item_s {
	int station;
	Macframe frame;
	long time_start;
	long time_header_start;
	long time_payload_start;
	long time_end;
	long time_collision;
	long time_corruption[SIM_STATION_COUNT];
} historyItem;

typedef struct marker_s {
	int station;
	int marker_type;
	long time;
	int added_to_list;
} Marker;

uint32_t *sim_left_buttons;
uint32_t *sim_right_buttons;
uint32_t *sim_up_buttons;
uint32_t *sim_middle_buttons;

#define RX_STATE_IDLE 0
#define RX_STATE_PREAMBLE 1
#define RX_STATE_PREAMBLE_CORRUPTED 2
#define RX_STATE_HEADER 3
#define RX_STATE_HEADER_CORRUPTED 4
#define RX_STATE_PAYLOAD 5
#define RX_STATE_PAYLOAD_CORRUPTED 6
#define RX_STATE_COMPLETE 7

#define TX_STATE_IDLE 0
#define TX_STATE_PREAMBLE 1
#define TX_STATE_PREAMBLE_CORRUPTED 2
#define TX_STATE_HEADER 3
#define TX_STATE_HEADER_CORRUPTED 4
#define TX_STATE_PAYLOAD 5
#define TX_STATE_PAYLOAD_CORRUPTED 6

uint32_t *sim_rx_states;
uint32_t *sim_tx_states;

#define RX_NO_HEADER 0
#define RX_GOOD_HEADER 1
#define RX_BAD_HEADER 2

uint32_t *sim_rx_header_indicators;

#define RX_NO_PAYLOAD 0
#define RX_GOOD_PAYLOAD 1
#define RX_BAD_PAYLOAD 2

uint32_t *sim_rx_payload_indicators;

#define TX_IDLE 0
#define TX_ACTIVE 1

uint32_t *sim_tx_active_indicators;

uint32_t *sim_rx_buffer_ids;
uint32_t *sim_tx_buffer_ids;

#define CS_BUSY 1
#define CS_IDLE 0

uint32_t *carrier_sense;

void **sim_packet_buffers;

#define HISTORY_SIZE 10000000
#define MARKERS_SIZE 1000000

int *history_last_item;
historyItem *history;
historyItem *station_current_history_items;

/*int *marker_last_item;
Marker *markers;*/

int *marker_last_items;
Marker **markers;


Marker *csma_marker;
int *csma_marker_last_item;

void setup_shared_memory();
void release_shared_memory();

typedef struct {
	int state_transition_matrix[9][9]; //Nine States in Total ([From][To])

	int ft_overheard;
	int recoveries[6]; //SameRing, SuccessorRing, Disappeared, Inconsistent, Timeout

	int token_ack[10]; //token was acked in the i-th retry
	int token_impl_ack[10]; //token was implicitly acked in the i-th retry
	int token_pass_fails; //token was not acked

	int token_received_wo_ft[10]; //same token was received and acked i times. ft was received
	int token_received_w_ft[10]; //same token was received and acked i times. ft was not received

	int data_ack[10];
	int data_fails[10];
	int data_rcvd_good_pl[10];
	int data_rcvd_bad_pl[10];
	int rings_created;
	int token_created;
	int rejoins;
	int rotations;
	int bad_header;
	int idle_timer_fired;
	int token_lifetimes[27];
	int pl_created;
	int packets_sent;
} Stats;

Stats *l_stats;

#endif
