/*
 * viewer.c
 *
 *  Created on: Aug 2, 2014
 *      Author: andreas
 */

#include "viewer.h"

void viewer_signal_handler(int signal);

static cairo_surface_t *surface = NULL;

GtkWidget *window;
GtkWidget *frame;
GtkWidget *da;
GtkWidget *box;
GtkWidget *list_box;
GtkListStore *list_store;

static void
clear_surface (void)
{
  cairo_t *cr;

  cr = cairo_create (surface);

  cairo_set_source_rgb (cr, 1, 1, 1);
  cairo_paint (cr);

  cairo_destroy (cr);
}

double scale = 1000 / SIM_SLOWDOWN;
double pan_x = 0;

void draw_marker(GtkWidget *widget, cairo_t *cr, long time, int station, double r, double g, double b, const char *s)
{
	double marker_time = (double)(time) / 1000000000 * 10 * scale + pan_x;

	if(marker_time > 0 && marker_time < gtk_widget_get_allocated_width(widget))
	{
		cairo_set_source_rgb (cr, r, g, b);

		cairo_move_to(cr, marker_time, 0);
		cairo_line_to(cr, marker_time, SIM_STATION_COUNT * 30 + 20);
		cairo_stroke(cr);
		cairo_move_to(cr, marker_time - 3, station * 30 + 20);
		cairo_line_to(cr, marker_time + 3, station * 30 + 20);
		cairo_stroke(cr);

		cairo_move_to (cr, marker_time, SIM_STATION_COUNT * 30 + 30);
		cairo_show_text (cr, s);
	}
}

static gboolean draw_cb (GtkWidget *widget, cairo_t *cr, gpointer data)
{
	GdkRGBA color;

	int i = 0;

	for(i=0; i<*history_last_item; ++i)
	{
		double start_packet = (double)(history[i].time_start) / 1000000000 * 10 * scale + pan_x;
		double end_packet = (double)(history[i].time_end) / 1000000000 * 10 * scale + pan_x;

		if(start_packet < gtk_widget_get_allocated_width(widget))
		{
			if(end_packet > 0)
			{
				double r,g,b;

				switch(history[i].frame.header.pktType)
				{
				case PACKET_TOKEN_FINAL:
					r = 0.5; g = 0; b = 0.5;
					break;
				case PACKET_TOKEN:
					r = 0; g = 0; b = 1;
					break;
				case PACKET_TOKEN_ACK:
					r = 0; g = 0.5; b = 1;
					break;
				case PACKET_CLAIM_TOKEN:
					r = 1; g = 0; b = 1;
					break;
				case PACKET_SOLICIT_SUCCESSOR:
					r = 0; g = 1; b = 1;
					break;
				case PACKET_DATA:
					r = 1; g = 1; b = 0;
					break;
				case PACKET_ACK:
					r = 0.5; g = 0.5; b = 0;
					break;
				case PACKET_SET_PREDECESSOR:
					r = 0; g = 0.5; b = 0;
					break;
				case PACKET_SET_PREDECESSOR_FINAL:
					r = 0.5; g = 0.5; b = 1;
					break;
				case PACKET_SET_SUCCESSOR:
					r = 0; g = 1; b = 0;
					break;
				case PACKET_SOLICIT_PREDECESSOR:
					r = 1; g = 0.5; b = 0.5;
					break;
				case PACKET_SOLICIT_PREDECESSOR_FINAL:
					r = 1; g = 0; b = 0.5;
					break;
				case PACKET_DUMMY_DATA:
					r = 0; g = 0.5; b = 1;
					break;
				default:
					printf("TYPE: %d\n", history[i].frame.header.pktType);
					r = 0; g = 0; b = 0;
					break;
				}

				cairo_set_source_rgb (cr, r, g, b);
				cairo_rectangle(cr, (double)(history[i].time_start) / 1000000000 * 10 * scale + pan_x, 10 + 30 * history[i].station, (double)(history[i].time_header_start - history[i].time_start) / 1000000000 * 10 * scale, 20);
				cairo_fill (cr);

				cairo_set_source_rgb (cr, r, g, b);
				cairo_rectangle(cr, (double)(history[i].time_header_start) / 1000000000 * 10 * scale + pan_x, 10 + 30 * history[i].station, (double)(history[i].time_payload_start - history[i].time_header_start) / 1000000000 * 10 * scale, 20);
				cairo_fill (cr);

				cairo_set_source_rgb (cr, r, g, b);
				cairo_rectangle(cr, (double)(history[i].time_payload_start) / 1000000000 * 10 * scale + pan_x, 10 + 30 * history[i].station, (double)(history[i].time_end - history[i].time_payload_start) / 1000000000 * 10 * scale, 20);
				cairo_fill (cr);

				/*if(history[i].time_collision > 0)
				{
					cairo_set_source_rgb (cr, 1, 0.8, 0.8);
					cairo_rectangle(cr, (double)(history[i].time_start) / 1000000000 * 10 * scale + pan_x, 5 + 30 * history[i].station, (double)(history[i].time_end - history[i].time_start) / 1000000000 * 10 * scale, 5);
					cairo_fill (cr);

					cairo_set_source_rgb (cr, 1, 0, 0);
					cairo_rectangle(cr, (double)(history[i].time_collision) / 1000000000 * 10 * scale + pan_x, 5 + 30 * history[i].station, (double)(history[i].time_end - history[i].time_collision) / 1000000000 * 10 * scale, 5);
					cairo_fill (cr);
				}*/

				int j;
				for(j=0; j<SIM_STATION_COUNT; ++j)
				{
					if(history[i].time_collision > 0 && j != history[i].station)
					{
						cairo_set_source_rgb (cr, 1, 0.8, 0.8);
						cairo_rectangle(cr, (double)(history[i].time_start) / 1000000000 * 10 * scale + pan_x, 5 + 30 * j, (double)(history[i].time_end - history[i].time_start) / 1000000000 * 10 * scale, 5);
						cairo_fill (cr);

						cairo_set_source_rgb (cr, 1, 0, 0);
						cairo_rectangle(cr, (double)(history[i].time_collision) / 1000000000 * 10 * scale + pan_x, 5 + 30 * j, (double)(history[i].time_end - history[i].time_collision) / 1000000000 * 10 * scale, 5);
						cairo_fill (cr);
					}
					if (history[i].time_collision <= 0 && j != history[i].station) //ms
					{
						cairo_set_source_rgb (cr, 0, 1.0, 0);
						cairo_rectangle(cr, (double)(history[i].time_start) / 1000000000 * 10 * scale + pan_x, 10 + 30 * j, (double)(history[i].time_end - history[i].time_start) / 1000000000 * 10 * scale, 5);
						cairo_fill (cr);
					}
					if(history[i].time_corruption[j] > 0)
					{
						cairo_set_source_rgb (cr, 1, 0.8, 0.8);
						cairo_rectangle(cr, (double)(history[i].time_start) / 1000000000 * 10 * scale + pan_x, 10 + 30 * j, (double)(history[i].time_end - history[i].time_start) / 1000000000 * 10 * scale, 5);
						cairo_fill (cr);

						cairo_set_source_rgb (cr, 1, 0, 0);
						cairo_rectangle(cr, (double)(history[i].time_corruption[j]) / 1000000000 * 10 * scale + pan_x, 10 + 30 * j, (double)(history[i].time_end - history[i].time_corruption[j]) / 1000000000 * 10 * scale, 5);
						cairo_fill (cr);
					}
				}
			}
		}







	}


	long markers_starts[SIM_STATION_COUNT];

	long csma_markers_starts[SIM_STATION_COUNT];

	double from;

	for(i=0; i<SIM_STATION_COUNT; ++i)
	{
		Marker *m = (Marker*)((void*)markers + i * MARKERS_SIZE * sizeof(Marker));

		int j;

		for(j=0; j<marker_last_items[i]; ++j)
		{
			double r,g,b;

			double marker_time = (double)(m[j].time) / 1000000000 * 10 * scale + pan_x;

			switch(m[j].marker_type)
			{
			case MARKER_HOLD_START:
				markers_starts[m[j].station] = m[j].time;
				break;
			case MARKER_HOLD_STOP:
				from = (double)(markers_starts[m[j].station]) / 1000000000 * 10 * scale + pan_x;

				if(from < gtk_widget_get_allocated_width(widget) && marker_time > 0)
				{
					cairo_set_source_rgb (cr, 0.7, 0.7, 0.7);
					cairo_rectangle(cr, from, 30 + 30 * m[j].station, marker_time - from, 5);
					cairo_fill (cr);
				}
				break;
			case MARKER_RECOVER:
				draw_marker(widget, cr, m[j].time, m[j].station, 1, 0, 0, "");
				break;
			case MARKER_IDLE_EXPIRED:
				draw_marker(widget, cr, m[j].time, m[j].station, 0, 0, 1, "");
				break;
			case MARKER_RECOVERY_STARTED:
				draw_marker(widget, cr, m[j].time, m[j].station, 0, 1, 1, "");
				break;
			case MARKER_RECOVERY_LOOP:
				draw_marker(widget, cr, m[j].time, m[j].station, 1, 0, 1, "L");
				break;
			case MARKER_RECOVERY_INCONSISTENT:
				draw_marker(widget, cr, m[j].time, m[j].station, 0, 1, 0, "I");

				if(m[j].added_to_list == 0)
				{
					GtkTreeIter iter;
					gtk_list_store_append (list_store, &iter);
					gtk_list_store_set (list_store, &iter,
											  0, m[j].time,
											  1, "Recovery Inconsistent",
											  -1);

					m[j].added_to_list = 1;
				}

				break;
			case MARKER_RECOVERY_DISAPPEAR:
				draw_marker(widget, cr, m[j].time, m[j].station, 1, 0, 1, "D");

				if(m[j].added_to_list == 0)
				{
					GtkTreeIter iter;
					gtk_list_store_append (list_store, &iter);
					gtk_list_store_set (list_store, &iter,
											  0, m[j].time,
											  1, "Recovery: Disappear",
											  -1);

					m[j].added_to_list = 1;
				}

				break;
			case MARKER_RECOVERY_SAMERING:
				draw_marker(widget, cr, m[j].time, m[j].station, 1, 0, 1, "R");

				if(m[j].added_to_list == 0)
								{
									GtkTreeIter iter;
									gtk_list_store_append (list_store, &iter);
									gtk_list_store_set (list_store, &iter,
															  0, m[j].time,
															  1, "Recovery: Same Ring",
															  -1);

									m[j].added_to_list = 1;
								}

				break;
			case MARKER_RECOVERY_SUCCESSORRING:
				draw_marker(widget, cr, m[j].time, m[j].station, 1, 0, 1, "S");

				if(m[j].added_to_list == 0)
								{
									GtkTreeIter iter;
									gtk_list_store_append (list_store, &iter);
									gtk_list_store_set (list_store, &iter,
															  0, m[j].time,
															  1, "Recovery: Successor Ring",
															  -1);

									m[j].added_to_list = 1;
								}

				break;
			case MARKER_RECOVERY_TIMEOUT:
				draw_marker(widget, cr, m[j].time, m[j].station, 1, 0, 1, "T");

				if(m[j].added_to_list == 0)
								{
									GtkTreeIter iter;
									gtk_list_store_append (list_store, &iter);
									gtk_list_store_set (list_store, &iter,
															  0, m[j].time,
															  1, "Recovery: Timeout",
															  -1);

									m[j].added_to_list = 1;
								}
				break;
			case MARKER_PREDECESSOR_MISMATCH:
				draw_marker(widget, cr, m[j].time, m[j].station, 0, 1, 0, "M");


				if(m[j].added_to_list == 0)
				{
					GtkTreeIter iter;
					gtk_list_store_append (list_store, &iter);
					gtk_list_store_set (list_store, &iter,
											  0, m[j].time,
											  1, "Predecessor Mismatch",
											  -1);

					m[j].added_to_list = 1;
				}
				break;
			case MARKER_MISC:
				draw_marker(widget, cr, m[j].time, m[j].station, 0, 0, 0, "1");
				break;
			case MARKER_MISC2:
				draw_marker(widget, cr, m[j].time, m[j].station, 0, 0, 0, "2");
				break;
			case MARKER_RECOVERY_DOUBLE_PICKUP:
				draw_marker(widget, cr, m[j].time, m[j].station, 1, 0, 1, "P");

				if(m[j].added_to_list == 0)
				{
					GtkTreeIter iter;
					gtk_list_store_append (list_store, &iter);
					gtk_list_store_set (list_store, &iter,
											  0, m[j].time,
											  1, "Recovery Double Pickup",
											  -1);

					m[j].added_to_list = 1;
				}
				break;



			case MARKER_RECOVERY_SAMERING_SP:
						draw_marker(widget, cr, m[j].time, m[j].station, 1, 0, 1, "R");

						if(m[j].added_to_list == 0)
						{
							GtkTreeIter iter;
							gtk_list_store_append (list_store, &iter);
							gtk_list_store_set (list_store, &iter,
													  0, m[j].time,
													  1, "Recovery Same Ring - Solicit Predecessor",
													  -1);

							m[j].added_to_list = 1;
						}
						break;

			case MARKER_IGNORED_T:
				draw_marker(widget, cr, m[j].time, m[j].station, 1, 0.5, 0, "T");

				if(m[j].added_to_list == 0)
				{
					GtkTreeIter iter;
					gtk_list_store_append (list_store, &iter);
					gtk_list_store_set (list_store, &iter,
											  0, m[j].time,
											  1, "Ignored Token",
											  -1);

					m[j].added_to_list = 1;
				}
				break;

			case MARKER_IGNORED_FT:
				draw_marker(widget, cr, m[j].time, m[j].station, 1, 0.5, 0, "F");

				if(m[j].added_to_list == 0)
				{
					GtkTreeIter iter;
					gtk_list_store_append (list_store, &iter);
					gtk_list_store_set (list_store, &iter,
											  0, m[j].time,
											  1, "Ignored Final Token",
											  -1);

					m[j].added_to_list = 1;
				}
				break;

			case MARKER_SILENCE_DETECTED:
				draw_marker(widget, cr, m[j].time, m[j].station, 0.54, 0.41, 0.8, "");

				if(m[j].added_to_list == 0)
				{
					GtkTreeIter iter;
					gtk_list_store_append (list_store, &iter);
					gtk_list_store_set (list_store, &iter,
											  0, m[j].time,
											  1, "Silence Detected",
											  -1);

					m[j].added_to_list = 1;
				}

				break;

			case MARKER_CSMA_FALLING_EDGE:
				csma_markers_starts[m[j].station] = m[j].time;
				break;

			case MARKER_CSMA_RISING_EDGE:
				from = (double)(csma_markers_starts[m[j].station]) / 1000000000 * 10 * scale + pan_x;

				if(from < gtk_widget_get_allocated_width(widget) && marker_time > 0)
				{
					//cairo_set_source_rgb (cr, 0.5, 1, 0.7);
					//cairo_rectangle(cr, from, 30 + 3 + 30 * m[j].station, marker_time - from, 5);
					//cairo_fill (cr);
				}
				break;
			case MARKER_BAD_HEADER:
							draw_marker(widget, cr, m[j].time, m[j].station, 0, 0, 0, "B");

							if(m[j].added_to_list == 0)
							{
								GtkTreeIter iter;
								gtk_list_store_append (list_store, &iter);
								gtk_list_store_set (list_store, &iter,
														  0, m[j].time,
														  1, "Bad Header Callback",
														  -1);

								m[j].added_to_list = 1;
							}

							break;
			default:
				break;
			}


		}
	}





	/*long current_time = get_time_nsec();

	if(*carrier_sense == CS_IDLE)
	{
		cairo_set_source_rgb (cr, 0, 0, 1);
		cairo_move_to(cr, (double)current_time * 10 / 1000000000  - 0.5, 5);
		cairo_line_to(cr, (double)current_time * 10 / 1000000000 + 0.5, 5);
		cairo_stroke(cr);
	}*/

	/*cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_FAST);
	cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);

	cairo_scale(cr, pan_x, 1);



	cairo_set_source_surface (cr, surface, 0, 0);
	  	  cairo_paint (cr);

	  	cairo_scale(cr, 1/pan_x, 1);


  cairo_arc (cr,
  	             100 / 2.0, 100 / 2.0,
  	             MIN (100, 100) / 2.0,
  	             0, 2 * G_PI);

  gtk_style_context_get_color (gtk_widget_get_style_context (widget),
                                   0,
                                   &color);
      gdk_cairo_set_source_rgba (cr, &color);

  	 cairo_fill (cr);*/

  	//





  return FALSE;
}

static gboolean configure_event_cb (GtkWidget *widget, GdkEventConfigure *event, gpointer data)
{
	if (surface)
		cairo_surface_destroy (surface);

	/*surface = gdk_window_create_similar_surface (gtk_widget_get_window (widget),
											   CAIRO_CONTENT_COLOR,
											   gtk_widget_get_allocated_width (widget),
											   gtk_widget_get_allocated_height (widget));
	*/
	surface = cairo_recording_surface_create(CAIRO_CONTENT_COLOR, NULL);

	/* Initialize the surface to white */
	clear_surface ();

	/* We've handled the configure event, no need for further processing. */
	return TRUE;
}

static void close_window (void)
{
	if (surface)
		cairo_surface_destroy (surface);

	gtk_main_quit ();
}

int transmission_painted[SIM_STATION_COUNT] = {0};

static gboolean
time_handler(GtkWidget *widget)
{



	double current_time = get_time_nsec();

	current_time /= 100000000L;



  cairo_t *cr;


  cr = cairo_create (surface);

  cairo_set_line_width (cr, 1.0);

  int i;
  for(i=0; i<SIM_STATION_COUNT; ++i)
  {
	  if(sim_tx_active_indicators[i])
	  {
		  cairo_move_to(cr, 0.5 + current_time,10 + 30*i);
		  cairo_line_to(cr, 0.5 + current_time,30 + 30*i);

		  cairo_stroke (cr);
	  }
  }


  cairo_destroy (cr);

  /* Now invalidate the affected region of the drawing area. */
  gtk_widget_queue_draw_area (widget, current_time, current_time + 1, 10, 30+30*SIM_STATION_COUNT);

  gtk_widget_queue_draw(widget);
  return TRUE;
}

double button_press_x = 0;
gboolean button_down = FALSE;

static gboolean
button_press_event_cb (GtkWidget      *widget,
                       GdkEventButton *event,
                       gpointer        data)
{
  /* paranoia check, in case we haven't gotten a configure event */
  if (surface == NULL)
    return FALSE;



  if (event->button == GDK_BUTTON_PRIMARY && event->type == GDK_BUTTON_PRESS)
    {

	  button_press_x = event->x;
	  button_down = TRUE;
      gtk_widget_queue_draw (widget);
    }

  if (event->button == GDK_BUTTON_PRIMARY && event->type == GDK_2BUTTON_PRESS)
  {
	  int station = -1;

	  int j;
	  for(j=0; j<SIM_STATION_COUNT; ++j)
	  {
		  if(event->y >= 10 + j * 30 && event->y <= 10 + j * 30 + 20)
		  {
			  station = j;
			  break;
		  }
	  }

	  if(station > -1)
	  {
		  long time = (event->x - pan_x) / scale / 10 * 1000000000;

		  int i;
		  while(history[i].station > -1)
		  {
			  if(history[i].time_start <= time && history[i].time_end >= time && history[i].station == station)
			  {
				  printf("Packet:\n");
				  printf("  Station: %d\n", history[i].station);
				  printf("  Time: %ld+%ld+%ld+%ld=%ld\n", history[i].time_start, history[i].time_header_start - history[i].time_start, history[i].time_payload_start - history[i].time_header_start, history[i].time_end - history[i].time_payload_start, history[i].time_end);
				  printf("----");

				  switch(history[i].frame.header.pktType)
				  	{
				  		case PACKET_TOKEN:
				  			printf("P_TOKEN\n");
				  			break;

				  		case PACKET_CLAIM_TOKEN:
				  			printf("P_CLAIM_TOKEN\n");
				  			break;

				  		case PACKET_SOLICIT_SUCCESSOR:
				  			printf("P_SOLICIT_SUCCESSOR\n");
				  			break;

				  		case PACKET_SET_PREDECESSOR:
				  			printf("P_SET_PREDECESSOR\n");
				  			break;

				  		case PACKET_SET_SUCCESSOR:
				  			printf("P_SET_SUCCESSOR\n");
				  			break;

				  		case PACKET_DATA:
				  			printf("PACKET_DATA\n");
				  			break;

				  		case PACKET_ACK:
				  			printf("PACKET_ACK\n");
				  			break;

				  		case PACKET_TOKEN_ACK:
				  			printf("PACKET_TOKEN_ACK\n");
				  			break;

				  		case PACKET_SET_PREDECESSOR_FINAL:
				  			printf("PACKET_SET_PREDECESSOR_FINAL\n");
				  			break;

				  		case PACKET_TOKEN_FINAL:
							printf("PACKET_TOKEN_FINAL\n");
							break;

				  		case PACKET_SOLICIT_PREDECESSOR:
				  			printf("PACKET_SOLICIT_PREDECESSOR\n");
				  			break;

				  		case PACKET_SOLICIT_PREDECESSOR_FINAL:
				  			printf("PACKET_SOLICIT_PREDECESSOR_FINAL\n");
				  			break;





				  		default:
				  			printf("%d\n", history[i].frame.header.pktType);
				  			break;
				  	}

				  /* ms printf("  addr: %d -> %d, next: %d \n", history[i].frame.header.srcAddr, history[i].frame.header.destAddr, history[i].frame.header.nextAddr);
				  printf("  length: %d\n", history[i].frame.header.length);
				  printf("  ring: (%d,%d), genSeq: %d, seq: %d\n", history[i].frame.header.ringAddr & 0xc0, history[i].frame.header.ringAddr & 0x3f, history[i].frame.header.genSeqNum, history[i].frame.header.seqNum);
				  printf("  nodes: %d\n", history[i].frame.header.nodes);
				  printf("  retx: %d\n", history[i].frame.header.retxCount);
				  printf("  tokenCreator: %d\n", history[i].frame.header.tokenCreator);
				  printf("  tokenCreatorPred: %d\n", history[i].frame.header.tokenCreatorPredecessor);
				  printf("  disableSolicit: %d\n", history[i].frame.header.disableSolicit);*/

				  break;
			  }

			  i++;
		  }

	  }

  }

  /* We've handled the event, stop processing */
  return TRUE;
}

static gboolean
button_release_event_cb (GtkWidget      *widget,
                       GdkEventButton *event,
                       gpointer        data)
{
  /* paranoia check, in case we haven't gotten a configure event */
  if (surface == NULL)
    return FALSE;



  if (event->button == GDK_BUTTON_PRIMARY)
    {

	  pan_x += event->x - button_press_x;

	  button_down = FALSE;

	   gtk_widget_queue_draw (widget);
    }


  /* We've handled the event, stop processing */
  return TRUE;
}

static gboolean
scroll_event_cb (GtkWidget      *widget,
                       GdkEventScroll *event,
                       gpointer        data)
{
	if(event->direction == GDK_SCROLL_UP)
	{
		scale *= 2;
		pan_x = pan_x * 2 - event->x;
	}
	else if(event->direction == GDK_SCROLL_DOWN)
	{
		scale /= 2;
		pan_x = (pan_x + event->x) / 2;

		if(scale < 1)
		{
			scale = 1;
		}
	}

	return TRUE;
}

static gboolean
motion_notify_event_cb (GtkWidget      *widget,
                       GdkEventMotion *event,
                       gpointer        data)
{
	if(button_down)
	{
		pan_x += event->x - button_press_x;
		button_press_x = event->x;
	}

	return TRUE;
}

void
   time_cell_data_function (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
   {
     glong  time;
     gchar   buf[40];

     gtk_tree_model_get(model, iter, 0, &time, -1);

     g_snprintf(buf, sizeof(buf), "%ld", time);

     g_object_set(renderer, "text", buf, NULL);
   }

void
   desc_cell_data_function (GtkTreeViewColumn *col,
                           GtkCellRenderer   *renderer,
                           GtkTreeModel      *model,
                           GtkTreeIter       *iter,
                           gpointer           user_data)
{
	gchar* str;
	gchar   buf[100];

	gtk_tree_model_get(model, iter, 1, &str, -1);

	g_snprintf(buf, sizeof(buf), "%s", str);

	g_object_set(renderer, "text", buf, NULL);
}

void
list_box_double_click_cb (GtkTreeView        *treeview,
                      GtkTreePath        *path,
                      GtkTreeViewColumn  *col,
                      gpointer            userdata)
 {
   GtkTreeModel *model;
   GtkTreeIter   iter;

   g_print ("A row has been double-clicked!\n");

   model = gtk_tree_view_get_model(treeview);

   if (gtk_tree_model_get_iter(model, &iter, path))
   {
      long time;

      gtk_tree_model_get(model, &iter, 0, &time, -1);

      scale = 8000 / SIM_SLOWDOWN ;

	  pan_x = -(double)(time) / 1000000000 * 10 * scale;

	  pan_x += gtk_widget_get_allocated_width(da) / 2;

   }
 }

int viewer_main()
{
	struct sigaction action;
	memset(&action, 0, sizeof(struct sigaction));
	action.sa_handler = viewer_signal_handler;
	sigaction(SIGTERM, &action, NULL);
	sigaction(SIGINT, &action, NULL);
	sigaction(SIGQUIT, &action, NULL);
	sigaction(SIGHUP, &action, NULL);

	printf("VIEWER_INFO: Hello World\n");



	gtk_init(NULL, NULL);

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);

	//gtk_window_set_title (GTK_WINDOW (window), "WarpSim");
#ifdef TRMAC_ETR_EXPL
	gtk_window_set_title (GTK_WINDOW (window), "WarpSim - TRMAC_ETR_EXPL");
#endif
#ifdef TRMAC_ETR_IMPL
	gtk_window_set_title (GTK_WINDOW (window), "WarpSim - TRMAC_ETR_IMPL");
#endif
#ifdef TRMAC_ETR_IMPL_CS
	gtk_window_set_title (GTK_WINDOW (window), "WarpSim - TRMAC_ETR_IMPL_CS");
#endif
#ifdef TRMAC_TDMA
	gtk_window_set_title (GTK_WINDOW (window), "WarpSim - TRMAC_TDMA");
#endif

	g_signal_connect (window, "destroy", G_CALLBACK (close_window), NULL);



	gtk_container_set_border_width (GTK_CONTAINER (window), 8);

	frame = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
	gtk_container_add (GTK_CONTAINER (window), frame);

	box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);


	gtk_box_set_homogeneous (GTK_BOX (box), TRUE);

	da = gtk_drawing_area_new ();
	gtk_box_pack_start(GTK_BOX (box), da, TRUE, TRUE, 0);

	list_box = gtk_tree_view_new();

	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;

	col = gtk_tree_view_column_new();

	gtk_tree_view_column_set_title(col, "Time");
	/* pack tree view column into tree view */
	gtk_tree_view_append_column(GTK_TREE_VIEW(list_box), col);
	renderer = gtk_cell_renderer_text_new();
	/* pack cell renderer into tree view column */
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func(col, renderer, time_cell_data_function, NULL, NULL);



	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Description");
	/* pack tree view column into tree view */
	gtk_tree_view_append_column(GTK_TREE_VIEW(list_box), col);
	renderer = gtk_cell_renderer_text_new();
	/* pack cell renderer into tree view column */
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func(col, renderer, desc_cell_data_function, NULL, NULL);


	list_store = gtk_list_store_new (2,
	                                   G_TYPE_LONG,
	                                   G_TYPE_STRING
	                                   );






	//gtk_tree_model_row_changed(GTK_TREE_MODEL(list_store), NULL, &iter);


	//gtk_tree_model_row_changed(GTK_TREE_MODEL(list_store), NULL, &iter);



	gtk_tree_view_set_model(GTK_TREE_VIEW(list_box), GTK_TREE_MODEL(list_store));
	g_signal_connect(list_box, "row-activated", (GCallback) list_box_double_click_cb, NULL);

	GtkWidget *scrolled_window;

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);



			gtk_container_add(GTK_CONTAINER (scrolled_window), list_box);

	gtk_box_pack_start(GTK_BOX (box), scrolled_window, TRUE, TRUE, 0);
	/* set a minimum size */

	gtk_widget_set_size_request (da, 600, 150);
	gtk_widget_set_size_request (list_box, 600, 150);
	gtk_container_add (GTK_CONTAINER (frame), box);

	/* Signals used to handle the backing surface */
	g_signal_connect (da, "draw",
					G_CALLBACK (draw_cb), NULL);
	g_signal_connect (da,"configure-event",
					G_CALLBACK (configure_event_cb), NULL);

	g_signal_connect (da, "button-press-event",
	                    G_CALLBACK (button_press_event_cb), NULL);

	g_signal_connect (da, "button-release-event",
		                    G_CALLBACK (button_release_event_cb), NULL);

	g_signal_connect (da, "scroll-event",
			                    G_CALLBACK (scroll_event_cb), NULL);

	g_signal_connect (da, "motion-notify-event",
				                    G_CALLBACK (motion_notify_event_cb), NULL);

	gtk_widget_set_events (da, gtk_widget_get_events (da)
	                             | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_SCROLL_MASK | GDK_POINTER_MOTION_MASK);

	gtk_widget_show_all (window);

	g_timeout_add(100, (GSourceFunc) time_handler, (gpointer) window);

	gtk_main ();

	return 0;
}

void viewer_signal_handler(int signal)
{
	const char *signal_name;

	switch (signal) {
		case SIGTERM:
			signal_name = "SIGTERM";
			break;
		case SIGQUIT:
			signal_name = "SIGQUIT";
			break;
		case SIGINT:
			signal_name = "SIGINT";
			break;
		case SIGHUP:
			signal_name = "SIGHUP";
			break;
		default:
			signal_name = "UNKNOWN_SIGNAL";
			break;
	}

	printf("VIEWER_INFO: Received %s, exiting...\n", signal_name);

	exit(0);
}
