/**
 * @file	network.h
 * @author	cd185990
 * @date	2010/07/02
 * @brief	Networking Structs / Defines
 *
 * Last Change Base: 		$LastChangedRevision: 161 $
 * Last Changed By: 		$LastChangedBy: cd185990 $
 * Last Changed On: 		$LastChangedDate: 2012-03-11 17:56:15 +0100 (Sun, 11 Mar 2012) $
 *
 * Copyright: 2010 - 2011, Mobile Network Performance Group, RWTH Aachen
 */

#ifndef NETWORK_H_
#define NETWORK_H_

#define NETWORK_VERSION	"0.2.0"

#define ETH_HDR_LENGTH	14
#define IP_HDR_LENGTH	20
#define UDP_HDR_LENGTH	8
#define TCP_HDR_LENGTH	20
#define ICMP_HDR_LENGTH	8

/**
 * Ethernet header.
 */
typedef struct ethhdr_t {
	unsigned char		eth_dest[6];	/* destination address */
	unsigned char		eth_src[6];		/* source address */
	unsigned short int	eth_proto;		/* ethernetType or length */
#define ETH_TYPE_IP4	0x0800
#define ETH_TYPE_IP6	0x86DD
#define ETH_TYPE_ARP	0x0806
#define ETH_TYPE_RARP	0x8035
#define ETH_TYPE_VLAN	0x8100
} __attribute__((packed)) ethhdr;


/**
 * IP header. // netinet/ip.h
 */
typedef struct iphdr_t {
#ifdef _IP_VHL
	unsigned char		ip_vhl;			/* version << 4 | header length >> 2 */
#else
//#if BYTE_ORDER == LITTLE_ENDIAN
//	unsigned int		ip_hl:4,		/* header length */
//						ip_v:4;			/* version */
//#endif
//#if BYTE_ORDER == BIG_ENDIAN
	unsigned int		ip_v:4,			/* version */
						ip_hl:4;		/* header length */
//#endif
#endif /* not _IP_VHL */
	unsigned char		ip_tos;			/* type of service */
#define	IP_TOS_LOWDELAY		0x10
#define	IP_TOS_THROUGHPUT	0x08
#define	IP_TOS_RELIABILITY	0x04
#define	IP_TOS_MINCOST		0x02
	unsigned short int	ip_len;			/* total length */
	unsigned short int	ip_id;			/* identification */
	unsigned short int	ip_off;			/* fragment offset field */
#define	IP_RF			0x8000			/* reserved fragment flag */
#define	IP_DF			0x4000			/* dont fragment flag */
#define	IP_MF			0x2000			/* more fragments flag */
#define	IP_OFFMASK		0x1fff			/* mask for fragmenting bits */
	unsigned char		ip_ttl;			/* time to live */
	unsigned char		ip_p;			/* protocol */
#define IP_P_ICMP		1
#define IP_P_IGMP		2
#define IP_P_TCP		6
#define IP_P_UDP		17
	unsigned short int	ip_sum;			/* checksum */
										/* source and dest address */
	struct { unsigned char s_b1,s_b2,s_b3,s_b4; } ip_src;
	struct { unsigned char s_b1,s_b2,s_b3,s_b4; } ip_dst;
} __attribute__((packed)) iphdr;


/**
 * UDP header. // netinet/udp.h
 */
typedef struct udphdr_t {
	unsigned short int	uh_sport;		/* source port */
	unsigned short int	uh_dport;		/* destination port */
	unsigned short int	uh_ulen;		/* udp length */
	unsigned short int	uh_sum;			/* udp checksum */
} __attribute__((packed)) udphdr;


/**
 * TCP header. // netinet/tcp.h
 */
typedef struct tcphdr_t {
	unsigned short int	th_sport;		/* source port */
	unsigned short int	th_dport;		/* destination port */
	unsigned int		th_seq;			/* sequence number */
	unsigned int		th_ack;			/* acknowledgement number */
//#if BYTE_ORDER == LITTLE_ENDIAN
//	unsigned int 		th_x2:4,		/* (unused) */
//						th_off:4;		/* data offset */
//#else
	unsigned int		th_off:4,		/* data offset */
						th_x2:4;		/* (unused) */
//#endif
	unsigned char		th_flags;
#define	TH_FIN	0x01
#define	TH_SYN	0x02
#define	TH_RST	0x04
#define	TH_PUSH	0x08
#define	TH_ACK	0x10
#define	TH_URG	0x20
#define	TH_ECE	0x40
#define	TH_CWR	0x80
#define	TH_FLAGS	(TH_FIN|TH_SYN|TH_RST|TH_ACK|TH_URG|TH_ECE|TH_CWR)
	unsigned short int	th_win;			/* window */
	unsigned short int	th_sum;			/* checksum */
	unsigned short int	th_urp;			/* urgent pointer */
} __attribute__((packed)) tcphdr;


/**
 * ICMP header. // netinet/icmp.h
 */
typedef struct icmphdr_t {
	unsigned char		icmp_type;		/* type of message, see below */
	unsigned char		icmp_code;		/* type sub code */
	unsigned short int	icmp_cksum;		/* ones complement cksum of struct */
	union {
		unsigned char	ih_pptr;		/* ICMP_PARAMPROB */
//		struct in_addr ih_gwaddr;		/* ICMP_REDIRECT */
		struct ih_idseq_t {
			unsigned short int	icd_id;
			unsigned short int	icd_seq;
		} ih_idseq;
		int ih_void;					/* ICMP_UNREACH_NEEDFRAG -- Path MTU Discovery (RFC1191) */
		struct ih_pmtu_t {
			unsigned short int	ipm_void;
			unsigned short int	ipm_nextmtu;
		} ih_pmtu;
		struct ih_rtradv_t {
			unsigned char irt_num_addrs;
			unsigned char irt_wpa;
			unsigned short int irt_lifetime;
		} ih_rtradv;
	} icmp_hun;
//	union {
//		struct id_ts_t {
//			n_time its_otime;
//			n_time its_rtime;
//			n_time its_ttime;
//		} id_ts;
//		struct id_ip_t  {
//			struct ip idi_ip;
//			/* options and then 64 bits of data */
//		} id_ip;
//		struct icmp_ra_addr id_radv;
//		u_int32_t id_mask;
//		char	id_data[1];
//	} icmp_dun;
#define	ICMP_ECHOREPLY		0			/* echo reply */
#define	ICMP_UNREACH		3			/* dest unreachable, codes: */
#define	ICMP_SOURCEQUENCH	4			/* packet lost, slow down */
#define	ICMP_REDIRECT		5			/* shorter route, codes: */
#define	ICMP_ECHO			8			/* echo service */
#define	ICMP_ROUTERADVERT	9			/* router advertisement */
#define	ICMP_ROUTERSOLICIT	10			/* router solicitation */
#define	ICMP_TIMXCEED		11			/* time exceeded, code: */
#define	ICMP_PARAMPROB		12			/* ip header bad */
#define	ICMP_TSTAMP			13			/* timestamp request */
#define	ICMP_TSTAMPREPLY	14			/* timestamp reply */
#define	ICMP_IREQ			15		/* information request */
#define	ICMP_IREQREPLY		16		/* information reply */
#define	ICMP_MASKREQ		17		/* address mask request */
#define	ICMP_MASKREPLY		18		/* address mask reply */
} __attribute__((packed)) icmphdr;


#endif /* NETWORK_H_ */
