
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include "warpphy.h"
#include "warpmac.h"
#include "../station.h"
#include "../settings.h"
#include "../helper.h"

//#define TIMER_DEBUG

void warpphy_clearRxPktStatus()
{
}

int warpphy_init(){
	xil_printf("  Initializing WARPPHY:\r\n");
	xil_printf("\tInitializing Radio Transmitter...\r\n");
	xil_printf("\tStarting TxDCO correction...\r\n");
	xil_printf("\tInitializing Radio Receiver...");
	xil_printf("complete!\r\n");
	xil_printf("\tInitializing Packet Detection...");
	xil_printf("complete!\r\n");
	xil_printf("\tInitializing OFDM Transmitter...");
	xil_printf("complete!\r\n");
	xil_printf("\tInitializing OFDM Receiver...");
	xil_printf("complete!\r\n");
	xil_printf("\tInitializing AGC...");
	xil_printf("complete!\r\n");
	/**********************************************************/

	/**************************Timer***************************/
	//Initialize the timer hardware; the individual timers will get setup in WARPMAC
	warp_timer_init();
	/**********************************************************/

	return 0;
}

void* warpphy_getBuffAddr(unsigned int c)
{
	if(c < SIM_PHY_BUFFER_COUNT)
	{
		return (void*)(((void*)station_packet_buffers) + c * SIM_PHY_BUFFER_SIZE);
	}
	else
	{
		printf("warpphy_getBuffAddr(unsigned int c): buffer not available.\n");
		return 0;
	}
}

///@brief Sets the packet buffer indices for the OFDM Tx and Rx PHY.
///
///The PLB_BRAM used a the PHY packet buffer is large enough to hold many PHY packets.
///This BRAM is divided into many sub-buffers; the PHY can be set to use any sub-buffer for Tx or Rx.
///
///@param txBufOffset 6-bit integer selecting the sub-buffer for the PHY Tx
///@param rxBufOffset 6-bit integer selecting the sub-buffer for the PHY Rx
void warpphy_setBuffs(unsigned char txBufOffset, unsigned char rxBufOffset){
	*station_tx_buffer_id = txBufOffset;
	*station_rx_buffer_id = rxBufOffset;

	//printf("Station %d: TxBuf: %d, RxBuf: %d\n", station_id, *station_tx_buffer_id, *station_rx_buffer_id);
}

int warpphy_pktTx(unsigned int block) {
	if(*station_tx_active_indicator)
	{
		return -1;
	}
	else
	{
		*station_tx_active_indicator = 1;

		/*ms
		printf("station % d transmits packet: ", station_id);
		print_packet_buffer(warpphy_getBuffAddr(*station_tx_buffer_id));*/

		return 0;
	}
}

int warpphy_txReady()
{
	return !(*station_tx_active_indicator);
}

int warpphy_waitForTx()
{
	while(*station_tx_active_indicator)
	{
		usleep(10);
	}

	return 0;
}





unsigned char warp_timer_isDone(unsigned char timer)
{
	return simTimers[timer].isDone;
}

unsigned char warp_timer_isActive(unsigned char timer)
{
	return simTimers[timer].isActive;
}

unsigned char warp_timer_isPaused(unsigned char timer)
{
	return simTimers[timer].isPaused;
}

//#define TIMER_DEBUG

void warp_timer_start(unsigned char timer)
{
	simTimers[timer].time_start = get_time_nsec();
	simTimers[timer].time_end = simTimers[timer].time_start + simTimers[timer].value;
	simTimers[timer].time_paused = 0;

	simTimers[timer].isActive = 1;
	simTimers[timer].isPaused = 0;
	simTimers[timer].isDone = 0;

#ifdef TIMER_DEBUG
	printf("Timer %d started. ", timer);
	printf("start=%lu, paused=%lu, end=%lu, value=%lu, a=%d, p=%d, d=%d\n",
					simTimers[timer].time_start,
					simTimers[timer].time_paused,
					simTimers[timer].time_end,
					simTimers[timer].value,
					simTimers[timer].isActive,
					simTimers[timer].isPaused,
					simTimers[timer].isDone);
#endif
}

void warp_timer_pause(unsigned char timer)
{
	if(simTimers[timer].isActive)
	{
		simTimers[timer].time_paused = get_time_nsec();

		simTimers[timer].isActive = 0;
		simTimers[timer].isPaused = 1;
		simTimers[timer].isDone = 0;
	}

#ifdef TIMER_DEBUG
	printf("Timer %d paused. ", timer);
	printf("start=%lu, paused=%lu, end=%lu, value=%lu, a=%d, p=%d, d=%d\n",
					simTimers[timer].time_start,
					simTimers[timer].time_paused,
					simTimers[timer].time_end,
					simTimers[timer].value,
					simTimers[timer].isActive,
					simTimers[timer].isPaused,
					simTimers[timer].isDone);
#endif

}

void warp_timer_resume(unsigned char timer)
{
	if(simTimers[timer].isPaused)
	{
		struct timespec ts;
	  	clock_gettime(CLOCK_REALTIME, &ts);

		long time_left = simTimers[timer].time_end - simTimers[timer].time_paused;

		simTimers[timer].time_start = get_time_nsec();
		simTimers[timer].time_end = simTimers[timer].time_start + time_left;
		simTimers[timer].time_paused = 0;

		simTimers[timer].isActive = 1;
		simTimers[timer].isPaused = 0;
		simTimers[timer].isDone = 0;
	}

#ifdef TIMER_DEBUG
	printf("Timer %d resumed. ", timer);
	printf("start=%lu, paused=%lu, end=%lu, value=%lu, a=%d, p=%d, d=%d\n",
					simTimers[timer].time_start,
					simTimers[timer].time_paused,
					simTimers[timer].time_end,
					simTimers[timer].value,
					simTimers[timer].isActive,
					simTimers[timer].isPaused,
					simTimers[timer].isDone);
#endif

}

void warp_timer_setMode(unsigned char timer, unsigned char mode)
{
	simTimers[timer].mode = mode;
}

void warp_timer_resetDone(unsigned char timer)
{
	simTimers[timer].isDone = 0;
}

void warp_timer_resetAllDoneStatus()
{
	int i;
	for(i=0; i<NUM_SIM_TIMERS; ++i)
	{
		simTimers[i].isDone = 0;
	}
}

void warp_timer_setTimer(unsigned char timer, unsigned int slotTime, unsigned int slotCount)
{
	if(slotTime != 0)
	{
		printf("warp_timer_setTimer(unsigned char timer, unsigned int slotTime, unsigned int slotCount) slotTime > 0 not supported.\n");
	}
	else
	{
		simTimers[timer].isActive = 0;
		simTimers[timer].isPaused = 0;
		simTimers[timer].isDone = 0;
		simTimers[timer].value = (long)slotCount / TIMERCLK_CYCLES_PER_USEC * 1000L * SIM_SLOWDOWN;

#ifdef TIMER_DEBUG
		printf("Timer %d set. ", timer);
		printf("start=%lu, paused=%lu, end=%lu, value=%lu, a=%d, p=%d, d=%d\n",
						simTimers[timer].time_start,
						simTimers[timer].time_paused,
						simTimers[timer].time_end,
						simTimers[timer].value,
						simTimers[timer].isActive,
						simTimers[timer].isPaused,
						simTimers[timer].isDone);
#endif
	}
}

void warp_timer_init()
{
	int i;
	for(i=0; i<NUM_SIM_TIMERS; ++i)
	{
		simTimers[i].time_start = 0;
		simTimers[i].time_paused = 0;
		simTimers[i].time_end = 0;
		simTimers[i].value = 0;

		simTimers[i].isActive = 0;		
		simTimers[i].isPaused = 0;		
		simTimers[i].isDone = 0;

		simTimers[i].mode = 0;
	}
}

int warpphy_setChannel(unsigned char band, unsigned int chan)
{
	int newFreq = -1;

	if (band == GHZ_2){
		switch(chan) {
			case(1) :
				newFreq = 2412;
				break;
			case(2) :
				newFreq = 2417;
				break;
			case(3) :
				newFreq = 2422;
				break;
			case(4) :
				newFreq = 2427;
				break;
			case(5) :
				newFreq = 2432;
				break;
			case(6) :
				newFreq = 2437;
				break;
			case(7) :
				newFreq = 2442;
				break;
			case(8) :
				newFreq = 2447;
				break;
			case(9) :
				newFreq = 2452;
				break;
			case(10) :
				newFreq = 2457;
				break;
			case(11) :
				newFreq = 2462;
				break;
			case(12) :
				newFreq = 2467;
				break;
			case(13) :
				newFreq = 2472;
				break;
			case(14) :
				newFreq = 2484;
				break;
			case(15) :
				newFreq = 2440;
				break;
			case(16) :
				newFreq = 2440;
				break;
			default :
				newFreq = -1;
		}
	}
	if (band == GHZ_5){
		switch(chan) {
			case(1) :
				newFreq = 5180;
				break;
			case(2) :
				newFreq = 5200;
				break;
			case(3) :
				newFreq = 5220;
				break;
			case(4) :
				newFreq = 5240;
				break;
			case(5) :
				newFreq = 5260;
				break;
			case(6) :
				newFreq = 5280;
				break;
			case(7) :
				newFreq = 5300;
				break;
			case(8) :
				newFreq = 5320;
				break;
			case(9) :
				newFreq = 5500;
				break;
			case(10) :
				newFreq = 5520;
				break;
			case(11) :
				newFreq = 5540;
				break;
			case(12) :
				newFreq = 5560;
				break;
			case(13) :
				newFreq = 5580;
				break;
			case(14) :
				newFreq = 5600;
				break;
			case(15) :
				newFreq = 5620;
				break;
			case(16) :
				newFreq = 5640;
				break;
			case(17) :
				newFreq = 5660;
				break;
			case(18) :
				newFreq = 5680;
				break;
			case(19) :
				newFreq = 5700;
				break;
			case(20) :
				newFreq = 5745;
				break;
			case(21) :
				newFreq = 5765;
				break;
			case(22) :
				newFreq = 5785;
				break;
			case(23) :
				newFreq = 5805;
				break;
			default :
				newFreq = -1;
		}
	}

	return newFreq;
}

//===================================================================================================================================================
//------NOT SCOPE OF SIMULATION-------
//===================================================================================================================================================



int warpphy_setAntennaMode(unsigned int txMode, unsigned int rxMode)
{
	return 0;
}

void warpphy_setLongCorrThresh(unsigned short thresh)
{
	return;
}

void warpphy_setEnergyDetThresh(unsigned short thresh)
{
	return;
}

void warpphy_setCarrierSenseThresh(unsigned short thresh)
{
	return;
}

void warpphy_setAutoCorrDetParams(unsigned short corrThresh, unsigned short energyThresh)
{
	return;
}

void warpphy_setAutoCorrMinDuration(unsigned char time)
{
	return;
}

int warpphy_setTxPower(unsigned char txPwr)
{
	if(txPwr > 63)
		return -1;
	else
		return 0;
}

//===================================================================================================================================================
//------UNCHANGED-------
//===================================================================================================================================================


//===================================================================================================================================================
//------NOT IMPLEMENTED-------
//===================================================================================================================================================

void warpphy_clearRxInterrupts()
{
	printf("warpphy_clearRxInterrupts() not implemented.\n");
}

void warpphy_clearTxInterrupts()
{
	printf("warpphy_clearTxInterrupts() not implemented.\n");
}

void mimo_ofdmRx_enable()
{
	printf("mimo_ofdmRx_enable() not implemented.\n");
}

void mimo_ofdmRx_disable()
{
	printf("mimo_ofdmRx_disable() not implemented.\n");
}

void mimo_ofdmRx_setOptions(unsigned int someOptions, unsigned int intType)
{
	printf("mimo_ofdmRx_setOptions(unsigned int someOptions, unsigned int intType) not implemented.\n");
}

unsigned int mimo_ofdmRx_getOptions()
{
	printf("int mimo_ofdmRx_getOptions() not implemented.\n");
	return 0;
}

void mimo_ofdmTx_disable()
{
	printf("mimo_ofdmTx_disable() not implemented.\n");
}

void mimo_ofdmTx_enable()
{
	printf("mimo_ofdmTx_enable() not implemented.\n");
}

void warpphy_setNumTrainingSyms(unsigned int numTraining)
{
	printf("warpphy_setNumTrainingSyms(unsigned int numTraining) not implemented.\n");
}

void warpphy_setPktDlyPlus()
{
	printf("warpphy_setPktDlyPlus() not implemented.\n");
}

void warpphy_setPktDlyMinus()
{
	printf("warpphy_setPktDlyMinus() not implemented.\n");
}

void warpphy_set_PN_KPlus(unsigned int increment)
{
	printf("warpphy_set_PN_KPlus(unsigned int increment) not implemented.\n");
}

void warpphy_set_PN_KMinus(unsigned int decrement)
{
	printf("warpphy_set_PN_KMinus(unsigned int decrement) not implemented.\n");
}

void warpphy_set_CFODebugOutput(unsigned char outputSel)
{
	printf("warpphy_set_CFODebugOutput(unsigned char outputSel) not implemented.\n");
}

void print_CFO_constants()
{
	printf("print_CFO_constants() not implemented.\n");
}

void warpphy_set_B_KPPlus(unsigned int increment)
{
	printf("warpphy_set_B_KPPlus(unsigned int increment) not implemented.\n");
}

void warpphy_set_B_KPMinus(unsigned int decrement)
{
	printf("warpphy_set_B_KPMinus(unsigned int decrement) not implemented.\n");
}

void warpphy_set_B_KIPlus(unsigned int increment)
{
	printf("warpphy_set_B_KIPlus(unsigned int increment) not implemented.\n");
}

void warpphy_set_B_KIMinus(unsigned int decrement)
{
	printf("warpphy_set_B_KIMinus(unsigned int decrement) not implemented.\n");
}

void warpphy_set_FFTOffset_Plus()
{
	printf("warpphy_set_FFTOffset_Plus() not implemented.\n");
}

void warpphy_set_FFTOffset_Minus()
{
	printf("warpphy_set_FFTOffset_Minus() not implemented.\n");
}

void warpphy_setNoiseTargetPlus()
{
	printf("warpphy_setNoiseTargetPlus() not implemented.\n");
}

void warpphy_setNoiseTargetMinus()
{
	printf("warpphy_setNoiseTargetMinus() not implemented.\n");
}

void warpphy_setTargetPlus()
{
	printf("warpphy_setTargetPlus() not implemented.\n");
}

void warpphy_setTargetMinus()
{
	printf("warpphy_setTargetMinus() not implemented.\n");
}

void warpphy_set_modulation(unsigned char baseRate, unsigned char TxAntAFullRate, unsigned char TxAntBFullRate, unsigned char RxAntAFullRate, unsigned char RxAntBFullRate)
{
	printf("warpphy_set_modulation(...) not implemented.\n");
}

int warpphy_applyTxDCOCorrection(unsigned int radioSelection)
{
	printf("warpphy_applyTxDCOCorrection(unsigned int radioSelection) not implemented.\n");
	return 0;
}

void warpphy_clearRxHeaderInterrupt()
{
	printf("warpphy_clearRxHeaderInterrupt() not implemented.\n");
}

void warpphy_setPktDetPlus(unsigned int offset)
{
	printf("warpphy_setPktDetPlus(unsigned int offset) not implemented.\n");
}

void warpphy_setPktDetMinus(unsigned int offset)
{
	printf("warpphy_setPktDetMinus(unsigned int offset) not implemented.\n");
}

void warpphy_setCSMAPlus(unsigned int offset)
{
	printf("warpphy_setCSMAPlus(unsigned int offset) not implemented.\n");
}

void warpphy_setCSMAMinus(unsigned int offset)
{
	printf("warpphy_setCSMAMinus(unsigned int offset) not implemented.\n");
}

int warpphy_isFree()
{
	printf("warpphy_isFree() not implemented.\n");
	return 0;
}

char warpphy_pollRxStatus(unsigned char type)
{
	printf("warpphy_pollRxStatus(unsigned char type) not implemented.\n");
	return 0;
}

void ofdm_AGC_SetTarget(unsigned int target)
{
	printf("ofdm_AGC_SetTarget(unsigned int target) not implemented.\n");
}

void ofdm_AGC_SetDCO(unsigned int AGCstate)
{
	printf("ofdm_AGC_SetDCO(unsigned int AGCstate) not implemented.\n");
}

void ofdm_AGC_Reset()
{
	printf("ofdm_AGC_Reset() not implemented.\n");
}

void ofdm_AGC_MasterReset()
{
	printf("ofdm_AGC_MasterReset() not implemented.\n");
}

void ofdm_AGC_Initialize(int noise_estimate)
{
	printf("ofdm_AGC_Initialize(int noise_estimate) not implemented.\n");
}

void ofdm_AGC_setNoiseEstimate(int noise_estimate)
{
	printf("ofdm_AGC_setNoiseEstimate(int noise_estimate) not implemented.\n");
}

unsigned int ofdm_AGC_GetGains(void)
{
	printf("ofdm_AGC_GetGains(void) not implemented.\n");
	return 0;
}

void ofdm_timer_start()
{
	printf("ofdm_timer_start() not implemented.\n");
}

void ofdm_timer_stop()
{
	printf("ofdm_timer_stop() not implemented.\n");
}

void ofdm_timer_clearInterrupt()
{
	printf("ofdm_timer_clearInterrupt() not implemented.\n");
}

void warpphy_incrementTxScaling(int incr_preamble, int incr_payload)
{
	printf("warpphy_incrementTxScaling(int incr_preamble, int incr_payload) not implemented.\n");
}

void warpphy_clearAutoResponseFlag(unsigned char flagID)
{
	printf("warpphy_clearAutoResponseFlag(unsigned char flagID) not implemented.\n");
}

void warpphy_setAntBPreambleShift(unsigned char shift)
{
	printf("warpphy_setAntBPreambleShift(unsigned char shift) not implemented.\n");
}

void warpphy_AFrecordEnable(unsigned char recordEn)
{
	printf("warpphy_AFrecordEnable(unsigned char recordEn) not implemented.\n");
}

void warpphy_setNumBaseRateSyms(unsigned int numSyms)
{
	printf("warpphy_setNumBaseRateSyms(unsigned int numSyms) not implemented.\n");
}



int warpphy_applyTxDCOCalibration()
{
	printf("warpphy_applyTxDCOCalibration() not implemented.\n");
	return 0;
}


//warp_timer function prototypes


unsigned char warp_timer_getStatus(unsigned char timer)
{
	printf("warp_timer_getStatus(unsigned char timer) not implemented.\n");
	return 0;
}


unsigned int warp_timer_getStatuses()
{
	printf("warp_timer_getStatuses() not implemented.\n");
	return 0;
}

unsigned char warp_timer_getDoneStatus()
{
	printf("warp_timer_getDoneStatus() not implemented.\n");
	return 0;
}

void warpphy_saveAFpkt(unsigned char doSave)
{
	printf("warpphy_saveAFpkt(unsigned char doSave) not implemented.\n");
}

int warpphy_setTxAntennaSwap(unsigned int txMode)
{
	printf("warpphy_setTxAntennaSwap(unsigned int txMode) not implemented.\n");
	return 0;
}

unsigned int warpphy_returnGainsDB()
{
	printf("warpphy_returnGainsDB() not implemented.\n");
	return 0;
}





